@@ -257,11 +257,12 @@ _router_process_peer_offer(
   struct dlep_router_if *router_if;
   union netaddr_socket local, remote;
   struct dlep_parser_value *value;
+  const struct os_interface_ip *ip;
   const struct netaddr *result = NULL;
   struct netaddr addr;
   uint16_t port;
   bool tls;
-  struct os_interface_data *ifdata;
+  struct os_interface *ifdata;
 
   if (session->restrict_signal != DLEP_PEER_OFFER) {
     /* ignore unless we are in discovery mode */
@@ -275,7 +276,7 @@ _router_process_peer_offer(
   result = NULL;
 
   /* remember interface data */
-  ifdata = &session->l2_listener.interface->data;
+  ifdata = session->l2_listener.data;
 
   /* IPv6 offer */
   value = dlep_session_get_tlv_value(session, DLEP_IPV6_CONPOINT_TLV);
@@ -290,8 +291,9 @@ _router_process_peer_offer(
     }
     else if (netaddr_is_in_subnet(&NETADDR_IPV6_LINKLOCAL, &addr)
         || result == NULL) {
-      result = oonf_interface_get_prefix_from_dst(&addr, ifdata);
-      if (result) {
+      ip = os_interface_get_prefix_from_dst(&addr, ifdata);
+      if (ip) {
+        result = &ip->address;
         netaddr_socket_init(&remote, &addr, port, ifdata->index);
       }
     }
@@ -309,8 +311,9 @@ _router_process_peer_offer(
       /* TLS not supported at the moment */
     }
     else {
-      result = oonf_interface_get_prefix_from_dst(&addr, ifdata);
-      if (result) {
+      ip = os_interface_get_prefix_from_dst(&addr, ifdata);
+      if (ip) {
+        result = &ip->address;
         netaddr_socket_init(&remote, &addr, port, ifdata->index);
       }
     }
@@ -320,13 +323,14 @@ _router_process_peer_offer(
   /* remote address of incoming session */
   if (!result) {
     netaddr_from_socket(&addr, &session->remote_socket);
-    result = oonf_interface_get_prefix_from_dst(&addr, ifdata);
-    if (!result) {
+    ip = os_interface_get_prefix_from_dst(&addr, ifdata);
+    if (!ip) {
       /* no possible way to communicate */
       OONF_DEBUG(session->log_source,
           "No matching prefix for incoming connection found");
       return -1;
     }
+    result = &ip->address;
     netaddr_socket_init(&remote, &addr, port, ifdata->index);
   }
 
