@@ -51,10 +51,10 @@
 #include "core/oonf_subsystem.h"
 #include "subsystems/oonf_class.h"
 #include "subsystems/oonf_clock.h"
-#include "subsystems/oonf_interface.h"
 #include "subsystems/oonf_layer2.h"
 #include "subsystems/oonf_rfc5444.h"
 #include "subsystems/oonf_timer.h"
+#include "subsystems/os_interface.h"
 
 #include "nhdp/nhdp_db.h"
 #include "nhdp/nhdp_interfaces.h"
@@ -128,10 +128,10 @@ static struct cfg_schema_section _probing_section = {
 static const char *_dependencies[] = {
   OONF_CLASS_SUBSYSTEM,
   OONF_CLOCK_SUBSYSTEM,
-  OONF_INTERFACE_SUBSYSTEM,
   OONF_LAYER2_SUBSYSTEM,
   OONF_RFC5444_SUBSYSTEM,
   OONF_TIMER_SUBSYSTEM,
+  OONF_OS_INTERFACE_SUBSYSTEM,
   OONF_NHDP_SUBSYSTEM,
 };
 static struct oonf_subsystem _olsrv2_neighbor_probing_subsystem = {
@@ -259,9 +259,9 @@ static void
 _cb_probe_link(struct oonf_timer_instance *ptr __attribute__((unused))) {
   struct nhdp_link *lnk, *best_lnk;
   struct _probing_link_data *ldata, *best_ldata;
-  struct nhdp_interface *ninterf;
+  struct nhdp_interface *nhdp_if;
 
-  struct os_interface *interf;
+  struct os_interface_listener *if_listener;
   struct oonf_layer2_net *l2net;
   struct oonf_layer2_neigh *l2neigh;
 
@@ -279,24 +279,24 @@ _cb_probe_link(struct oonf_timer_instance *ptr __attribute__((unused))) {
 
   l2neigh = NULL;
 
-  avl_for_each_element(nhdp_interface_get_tree(), ninterf, _node) {
-    interf = nhdp_interface_get_coreif(ninterf);
+  avl_for_each_element(nhdp_interface_get_tree(), nhdp_if, _node) {
+    if_listener = nhdp_interface_get_if_listener(nhdp_if);
 
-    l2net = oonf_layer2_net_get(interf->data.name);
+    l2net = oonf_layer2_net_get(if_listener->data->name);
     if (!l2net) {
       continue;
     }
 
     if(!_check_if_type(l2net)) {
       OONF_DEBUG(LOG_PROBING, "Drop interface %s (not wireless)",
-          interf->data.name);
+          if_listener->data->name);
       continue;
     }
 
     OONF_DEBUG(LOG_PROBING, "Start looking for probe candidate in interface '%s'",
-        interf->data.name);
+        if_listener->data->name);
 
-    list_for_each_element(&ninterf->_links, lnk, _if_node) {
+    list_for_each_element(&nhdp_if->_links, lnk, _if_node) {
       if (lnk->status != NHDP_LINK_SYMMETRIC) {
         /* only probe symmetric neighbors */
         continue;
