@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -49,11 +53,11 @@
 #include "core/oonf_logging.h"
 #include "core/oonf_subsystem.h"
 #include "subsystems/oonf_class.h"
-#include "subsystems/oonf_interface.h"
 #include "subsystems/oonf_timer.h"
 #include "subsystems/oonf_stream_socket.h"
+#include "subsystems/os_interface.h"
 #include "subsystems/os_system.h"
-#include "subsystems/os_socket.h"
+#include "subsystems/os_fd.h"
 
 /* Definitions */
 #define LOG_STREAM _oonf_stream_socket_subsystem.logging
@@ -65,16 +69,16 @@ static void _cleanup(void);
 static void _stream_close(struct oonf_stream_session *session);
 int _apply_managed(struct oonf_stream_managed *managed);
 static int _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
-    struct oonf_stream_socket *stream, struct os_interface_data *data);
-static void _cb_parse_request(int fd, void *data, bool, bool);
+    struct oonf_stream_socket *stream, struct os_interface *os_if);
+static void _cb_parse_request(struct oonf_socket_entry *);
 static struct oonf_stream_session *_create_session(
-    struct oonf_stream_socket *stream_socket, int sock,
+    struct oonf_stream_socket *stream_socket, struct os_fd *sock,
     const struct netaddr *remote_addr,
     const union netaddr_socket *remote_socket);
-static void _cb_parse_connection(int fd, void *data, bool r,bool w);
+static void _cb_parse_connection(struct oonf_socket_entry *entry);
 
-static void _cb_timeout_handler(void *);
-static int _cb_interface_listener(struct oonf_interface_listener *l);
+static void _cb_timeout_handler(struct oonf_timer_instance *);
+static int _cb_interface_listener(struct os_interface_listener *listener);
 
 /* list of olsr stream sockets */
 static struct list_entity _stream_head;
@@ -93,11 +97,11 @@ static struct oonf_timer_class _connection_timeout = {
 /* subsystem definition */
 static const char *_dependencies[] = {
   OONF_CLASS_SUBSYSTEM,
-  OONF_INTERFACE_SUBSYSTEM,
   OONF_SOCKET_SUBSYSTEM,
   OONF_TIMER_SUBSYSTEM,
+  OONF_OS_FD_SUBSYSTEM,
+  OONF_OS_INTERFACE_SUBSYSTEM,
   OONF_OS_SYSTEM_SUBSYSTEM,
-  OONF_OS_SOCKET_SUBSYSTEM,
 };
 
 static struct oonf_subsystem _oonf_stream_socket_subsystem = {
@@ -158,30 +162,27 @@ oonf_stream_flush(struct oonf_stream_session *con) {
 int
 oonf_stream_add(struct oonf_stream_socket *stream_socket,
     const union netaddr_socket *local) {
-  int s = -1;
   struct netaddr_str buf;
 
   /* server socket not necessary for outgoing connections */
   if (netaddr_socket_get_port(local) != 0) {
     /* Init socket */
-    s = os_socket_getsocket(local, true, 0, NULL, LOG_STREAM);
-    if (s < 0) {
+    if (os_fd_getsocket(&stream_socket->scheduler_entry.fd,
+        local, true, 0, NULL, LOG_STREAM)) {
       goto add_stream_error;
     }
 
     /* show that we are willing to listen */
-    if (os_socket_listen(s, 1) == -1) {
+    if (os_fd_listen(&stream_socket->scheduler_entry.fd, 1) == -1) {
       OONF_WARN(LOG_STREAM, "tcp socket listen failed for %s: %s (%d)\n",
           netaddr_socket_to_string(&buf, local), strerror(errno), errno);
       goto add_stream_error;
     }
 
-    stream_socket->scheduler_entry.fd = s;
     stream_socket->scheduler_entry.process = _cb_parse_request;
-    stream_socket->scheduler_entry.data = stream_socket;
-    stream_socket->scheduler_entry.event_read = true;
 
     oonf_socket_add(&stream_socket->scheduler_entry);
+    oonf_socket_set_read(&stream_socket->scheduler_entry, true);
   }
   memcpy(&stream_socket->local_socket, local, sizeof(stream_socket->local_socket));
 
@@ -201,12 +202,8 @@ oonf_stream_add(struct oonf_stream_socket *stream_socket,
   return 0;
 
 add_stream_error:
-  if (stream_socket->scheduler_entry.fd) {
-    oonf_socket_remove(&stream_socket->scheduler_entry);
-  }
-  if (s != -1) {
-    os_socket_close(s);
-  }
+  oonf_socket_remove(&stream_socket->scheduler_entry);
+  os_fd_close(&stream_socket->scheduler_entry.fd);
   return -1;
 }
 
@@ -230,11 +227,8 @@ oonf_stream_remove(struct oonf_stream_socket *stream_socket, bool force) {
   oonf_stream_close_all_sessions(stream_socket);
   list_remove(&stream_socket->_node);
 
-  if (stream_socket->scheduler_entry.fd) {
-    /* only for server sockets */
-    os_socket_close(stream_socket->scheduler_entry.fd);
-    oonf_socket_remove(&stream_socket->scheduler_entry);
-  }
+  oonf_socket_remove(&stream_socket->scheduler_entry);
+  os_fd_close(&stream_socket->scheduler_entry.fd);
 }
 
 /**
@@ -269,9 +263,9 @@ struct oonf_stream_session *
 oonf_stream_connect_to(struct oonf_stream_socket *stream_socket,
     const union netaddr_socket *remote) {
   struct oonf_stream_session *session;
+  struct os_fd sock;
   struct netaddr remote_addr;
   bool wait_for_connect = false;
-  int s;
   struct netaddr_str nbuf1;
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str nbuf2;
@@ -281,13 +275,12 @@ oonf_stream_connect_to(struct oonf_stream_socket *stream_socket,
       netaddr_socket_to_string(&nbuf1, &stream_socket->local_socket),
       netaddr_socket_to_string(&nbuf2, remote));
 
-  s = os_socket_getsocket(&stream_socket->local_socket,
-      true, 0, NULL, LOG_STREAM);
-  if (s < 0) {
+  if (os_fd_getsocket(&sock,
+      &stream_socket->local_socket, true, 0, NULL, LOG_STREAM)) {
     return NULL;
   }
 
-  if (os_socket_connect(s, remote)) {
+  if (os_fd_connect(&sock, remote)) {
     if (errno != EINPROGRESS) {
       OONF_WARN(LOG_STREAM, "Cannot connect outgoing tcp connection to %s: %s (%d)",
           netaddr_socket_to_string(&nbuf1, remote), strerror(errno), errno);
@@ -297,7 +290,7 @@ oonf_stream_connect_to(struct oonf_stream_socket *stream_socket,
   }
 
   netaddr_from_socket(&remote_addr, remote);
-  session = _create_session(stream_socket, s, &remote_addr, remote);
+  session = _create_session(stream_socket, &sock, &remote_addr, remote);
   if (session) {
     session->wait_for_connect = wait_for_connect;
     return session;
@@ -305,9 +298,7 @@ oonf_stream_connect_to(struct oonf_stream_socket *stream_socket,
 
   /* fall through */
 connect_to_error:
-  if (s) {
-    os_socket_close(s);
-  }
+  os_fd_close(&stream_socket->scheduler_entry.fd);
   return NULL;
 }
 
@@ -324,8 +315,6 @@ oonf_stream_set_timeout(struct oonf_stream_session *con, uint64_t timeout) {
 /**
  * Close a TCP stream session
  * @param session pointer to stream session
- * @param force true if the session should be closed instantly,
- *   false if all data in queue should still be sent
  */
 void
 oonf_stream_close(struct oonf_stream_session *session) {
@@ -353,7 +342,7 @@ oonf_stream_add_managed(struct oonf_stream_managed *managed) {
     managed->config.session_timeout = 120000;
   }
 
-  managed->_if_listener.process = _cb_interface_listener;
+  managed->_if_listener.if_changed = _cb_interface_listener;
   managed->_if_listener.name = managed->_managed_config.interface;
 }
 
@@ -382,10 +371,10 @@ oonf_stream_apply_managed(struct oonf_stream_managed *managed,
   /* handle change in interface listener */
   if (if_changed) {
     /* interface changed, remove old listener if necessary */
-    oonf_interface_remove_listener(&managed->_if_listener);
+    os_interface_remove(&managed->_if_listener);
 
     /* create new interface listener */
-    oonf_interface_add_listener(&managed->_if_listener);
+    os_interface_add(&managed->_if_listener);
   }
 
   OONF_DEBUG(LOG_STREAM, "Apply changes for managed socket (if %s) with port %d",
@@ -395,7 +384,7 @@ oonf_stream_apply_managed(struct oonf_stream_managed *managed,
   result = _apply_managed(managed);
   if (result) {
     /* did not work, trigger interface handler to try later again */
-    oonf_interface_trigger_handler(&managed->_if_listener);
+    os_interface_trigger_handler(&managed->_if_listener);
   }
   return result;
 }
@@ -408,11 +397,11 @@ oonf_stream_apply_managed(struct oonf_stream_managed *managed,
  */
 void
 oonf_stream_remove_managed(struct oonf_stream_managed *managed, bool force) {
-  oonf_interface_remove_listener(&managed->_if_listener);
+  os_interface_remove(&managed->_if_listener);
 
   oonf_stream_remove(&managed->socket_v4, force);
   oonf_stream_remove(&managed->socket_v6, force);
-  oonf_interface_remove_listener(&managed->_if_listener);
+  os_interface_remove(&managed->_if_listener);
   oonf_stream_free_managed_config(&managed->_managed_config);
 }
 
@@ -465,22 +454,22 @@ _stream_close(struct oonf_stream_session *session) {
     return;
   }
 
-  if (session->comport->config.cleanup) {
-    session->comport->config.cleanup(session);
+  if (session->stream_socket->config.cleanup) {
+    session->stream_socket->config.cleanup(session);
   }
 
   oonf_timer_stop(&session->timeout);
 
-  session->comport->config.allowed_sessions++;
+  session->stream_socket->session_counter--;
   list_remove(&session->node);
 
-  os_socket_close(session->scheduler_entry.fd);
   oonf_socket_remove(&session->scheduler_entry);
+  os_fd_close(&session->scheduler_entry.fd);
 
   abuf_free(&session->in);
   abuf_free(&session->out);
 
-  oonf_class_free(session->comport->config.memcookie, session);
+  oonf_class_free(session->stream_socket->config.memcookie, session);
 }
 
 /**
@@ -490,19 +479,19 @@ _stream_close(struct oonf_stream_session *session) {
  */
 int
 _apply_managed(struct oonf_stream_managed *managed) {
-  struct os_interface_data *data = NULL;
+  struct os_interface *bind_socket_to_if = NULL;
 
   /* get interface */
-  if (managed->_if_listener.interface) {
-    data = &managed->_if_listener.interface->data;
+  if (!managed->_if_listener.data->flags.any) {
+    bind_socket_to_if = managed->_if_listener.data;
   }
 
-  if (_apply_managed_socket(AF_INET, managed, &managed->socket_v4, data)) {
+  if (_apply_managed_socket(AF_INET, managed, &managed->socket_v4, bind_socket_to_if)) {
     return -1;
   }
 
   if (os_system_is_ipv6_supported()) {
-    if (_apply_managed_socket(AF_INET6, managed, &managed->socket_v6, data)) {
+    if (_apply_managed_socket(AF_INET6, managed, &managed->socket_v6, bind_socket_to_if)) {
       return -1;
     }
   }
@@ -518,7 +507,7 @@ _apply_managed(struct oonf_stream_managed *managed) {
  */
 static int
 _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
-    struct oonf_stream_socket *stream, struct os_interface_data *data) {
+    struct oonf_stream_socket *stream, struct os_interface *data) {
   struct netaddr_acl *bind_ip_acl;
   const struct netaddr *bind_ip;
   union netaddr_socket sock;
@@ -527,16 +516,17 @@ _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
   bind_ip_acl = &managed->_managed_config.bindto;
 
   /* Get address the unicast socket should bind on */
-  if (data != NULL && !data->up) {
+  if (data != NULL && !data->flags.up) {
     bind_ip = NULL;
   }
-  else if (data != NULL && netaddr_get_address_family(data->linklocal_v6_ptr) == af_type &&
-      netaddr_acl_check_accept(bind_ip_acl, data->linklocal_v6_ptr)) {
+  else if (data != NULL
+      && netaddr_get_address_family(data->if_linklocal_v6) == af_type
+      && netaddr_acl_check_accept(bind_ip_acl, data->if_linklocal_v6)) {
 
-    bind_ip = data->linklocal_v6_ptr;
+    bind_ip = data->if_linklocal_v6;
   }
   else {
-    bind_ip = oonf_interface_get_bindaddress(af_type, bind_ip_acl, data);
+    bind_ip = os_interface_get_bindaddress(af_type, bind_ip_acl, data);
   }
   if (!bind_ip) {
     oonf_stream_remove(stream, true);
@@ -551,7 +541,8 @@ _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
 
   if (list_is_node_added(&stream->scheduler_entry._node)) {
     if (memcmp(&sock, &stream->local_socket, sizeof(sock)) == 0) {
-      /* nothing changed */
+      /* nothing changed, just copy configuration */
+      memcpy(&stream->config, &managed->config, sizeof(stream->config));
       return 0;
     }
 
@@ -571,32 +562,25 @@ _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
 
 /**
  * Handle incoming server socket event from socket scheduler.
- * @param fd filedescriptor for event
- * @param data custom user data
- * @param event_read true if read-event is incoming
- * @param event_write true if write-event is incoming
+ * @param entry socket entry for event parsing
  */
 static void
-_cb_parse_request(int fd, void *data, bool event_read,
-    bool event_write __attribute__((unused))) {
+_cb_parse_request(struct oonf_socket_entry *entry) {
   struct oonf_stream_socket *stream;
   union netaddr_socket remote_socket;
   struct netaddr remote_addr;
-  socklen_t addrlen;
-  int sock;
+  struct os_fd sock;
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str buf1, buf2;
 #endif
 
-  if (!event_read) {
+  if (!oonf_socket_is_read(entry)) {
     return;
   }
 
-  stream = data;
+  stream = container_of(entry, typeof(*stream), scheduler_entry);
 
-  addrlen = sizeof(remote_socket);
-  sock = accept(fd, &remote_socket.std, &addrlen);
-  if (sock < 0) {
+  if (os_fd_accept(&sock, &entry->fd, &remote_socket)) {
     OONF_WARN(LOG_STREAM, "accept() call returned error: %s (%d)", strerror(errno), errno);
     return;
   }
@@ -607,11 +591,11 @@ _cb_parse_request(int fd, void *data, bool event_read,
       OONF_DEBUG(LOG_STREAM, "Access from %s to socket %s blocked because of ACL",
           netaddr_to_string(&buf1, &remote_addr),
           netaddr_socket_to_string(&buf2, &stream->local_socket));
-      close(sock);
+      os_fd_close(&sock);
       return;
     }
   }
-  _create_session(stream, sock, &remote_addr, &remote_socket);
+  _create_session(stream, &sock, &remote_addr, &remote_socket);
 }
 
 /**
@@ -623,7 +607,7 @@ _cb_parse_request(int fd, void *data, bool event_read,
  */
 static struct oonf_stream_session *
 _create_session(struct oonf_stream_socket *stream_socket,
-    int sock, const struct netaddr *remote_addr,
+    struct os_fd *sock, const struct netaddr *remote_addr,
     const union netaddr_socket *remote_socket) {
   struct oonf_stream_session *session;
 #ifdef OONF_LOG_DEBUG_INFO
@@ -631,7 +615,7 @@ _create_session(struct oonf_stream_socket *stream_socket,
 #endif
 
   /* put socket into non-blocking mode */
-  if (os_socket_set_nonblocking(sock)) {
+  if (os_fd_set_nonblocking(sock)) {
     OONF_WARN(LOG_STREAM, "Cannot read comport socket status: %s (%d)",
         strerror(errno), errno);
     return NULL;
@@ -652,24 +636,21 @@ _create_session(struct oonf_stream_socket *stream_socket,
     goto parse_request_error;
   }
 
-  session->scheduler_entry.fd = sock;
+  os_fd_copy(&session->scheduler_entry.fd, sock);
   session->scheduler_entry.process = _cb_parse_connection;
-  session->scheduler_entry.data = session;
-  session->scheduler_entry.event_read = true;
-  session->scheduler_entry.event_write = true;
   oonf_socket_add(&session->scheduler_entry);
-
+  oonf_socket_set_read(&session->scheduler_entry, true);
+  oonf_socket_set_write(&session->scheduler_entry, true);
   session->send_first = stream_socket->config.send_first;
-  session->comport = stream_socket;
+  session->stream_socket = stream_socket;
 
   session->remote_address = *remote_addr;
   session->remote_socket = *remote_socket;
 
-  session->copy_fd = -1;
-
-  if (stream_socket->config.allowed_sessions-- > 0) {
+  if (stream_socket->session_counter < stream_socket->config.allowed_sessions) {
     /* create active session */
     session->state = STREAM_SESSION_ACTIVE;
+    stream_socket->session_counter++;
   } else {
     /* too many sessions */
     if (stream_socket->config.create_error) {
@@ -678,7 +659,6 @@ _create_session(struct oonf_stream_socket *stream_socket,
     session->state = STREAM_SESSION_SEND_AND_QUIT;
   }
 
-  session->timeout.cb_context = session;
   session->timeout.class = &_connection_timeout;
   if (stream_socket->config.session_timeout) {
     oonf_timer_start(&session->timeout, stream_socket->config.session_timeout);
@@ -691,7 +671,7 @@ _create_session(struct oonf_stream_socket *stream_socket,
   }
 
   OONF_DEBUG(LOG_STREAM, "Got connection through socket %d with %s.\n",
-      sock, netaddr_to_string(&buf, remote_addr));
+      os_fd_get_fd(sock), netaddr_to_string(&buf, remote_addr));
 
   list_add_tail(&stream_socket->session, &session->node);
   return session;
@@ -706,11 +686,13 @@ parse_request_error:
 
 /**
  * Handle TCP session timeout
- * @param data custom data
+ * @param ptr timer instance that fired
  */
 static void
-_cb_timeout_handler(void *data) {
-  struct oonf_stream_session *session = data;
+_cb_timeout_handler(struct oonf_timer_instance *ptr) {
+  struct oonf_stream_session *session;
+
+  session = container_of(ptr, struct oonf_stream_session, timeout);
   oonf_stream_close(session);
 }
 
@@ -722,27 +704,28 @@ _cb_timeout_handler(void *data) {
  * @param event_write true if write-event is incoming
  */
 static void
-_cb_parse_connection(int fd, void *data, bool event_read, bool event_write) {
+_cb_parse_connection(struct oonf_socket_entry *entry) {
   struct oonf_stream_session *session;
   struct oonf_stream_socket *s_sock;
   int len;
   char buffer[1024];
   struct netaddr_str buf;
 
-  session = data;
-  s_sock = session->comport;
+  session = container_of(entry, typeof(*session), scheduler_entry);
+  s_sock = session->stream_socket;
 
-  OONF_DEBUG(LOG_STREAM, "Parsing connection of socket %d\n", fd);
+  OONF_DEBUG(LOG_STREAM, "Parsing connection of socket %d\n",
+      os_fd_get_fd(&entry->fd));
 
   /* mark session and s_sock as busy */
   session->busy = true;
   s_sock->busy = true;
 
   if (session->wait_for_connect) {
-    if (event_write) {
+    if (oonf_socket_is_write(entry)) {
       int value;
 
-      if(os_socket_get_socket_error(fd, &value)) {
+      if(os_fd_get_socket_error(&entry->fd, &value)) {
         OONF_WARN(LOG_STREAM, "getsockopt failed: %s (%d)",
             strerror(errno), errno);
         session->state = STREAM_SESSION_CLEANUP;
@@ -766,8 +749,8 @@ _cb_parse_connection(int fd, void *data, bool event_read, bool event_write) {
   }
 
   /* read data if necessary */
-  if (session->state == STREAM_SESSION_ACTIVE && event_read) {
-    len = os_socket_recvfrom(fd, buffer, sizeof(buffer), NULL, 0);
+  if (session->state == STREAM_SESSION_ACTIVE && oonf_socket_is_read(entry)) {
+    len = os_fd_recvfrom(&entry->fd, buffer, sizeof(buffer), NULL, 0);
     if (len > 0) {
       OONF_DEBUG(LOG_STREAM, "  recv returned %d\n", len);
       if (abuf_memcpy(&session->in, buffer, len)) {
@@ -793,6 +776,12 @@ _cb_parse_connection(int fd, void *data, bool event_read, bool event_write) {
     } else if (len == 0) {
       /* external s_sock closed */
       session->state = STREAM_SESSION_SEND_AND_QUIT;
+
+      /* still call callback once more */
+      session->state = s_sock->config.receive_data(session);
+
+      /* switch off read events */
+      oonf_socket_set_read(entry, false);
     }
   }
 
@@ -804,8 +793,8 @@ _cb_parse_connection(int fd, void *data, bool event_read, bool event_write) {
 
   /* send data if necessary */
   if (session->state != STREAM_SESSION_CLEANUP && abuf_getlen(&session->out) > 0) {
-    if (event_write) {
-      len = os_socket_sendto(fd, abuf_getptr(&session->out), abuf_getlen(&session->out), NULL, false);
+    if (oonf_socket_is_write(entry)) {
+      len = os_fd_sendto(&entry->fd, abuf_getptr(&session->out), abuf_getlen(&session->out), NULL, false);
 
       if (len > 0) {
         OONF_DEBUG(LOG_STREAM, "  send returned %d\n", len);
@@ -825,13 +814,15 @@ _cb_parse_connection(int fd, void *data, bool event_read, bool event_write) {
 
   /* send file if necessary */
   if (session->state == STREAM_SESSION_SEND_AND_QUIT
-      && abuf_getlen(&session->out) == 0 && session->copy_fd != -1) {
-    if (event_write) {
-      len = os_socket_sendfile(fd, session->copy_fd, session->copy_bytes_sent,
+      && abuf_getlen(&session->out) == 0
+      && os_fd_is_initialized(&session->copy_fd)) {
+    if (oonf_socket_is_write(entry)) {
+      len = os_fd_sendfile(&entry->fd, &session->copy_fd, session->copy_bytes_sent,
           session->copy_total_size - session->copy_bytes_sent);
       if (len <= 0) {
         OONF_WARN(LOG_STREAM, "Error while copying file to output stream (%d/%d): %s (%d)",
-            fd, session->copy_fd, strerror(errno), errno);
+            os_fd_get_fd(&entry->fd),
+            os_fd_get_fd(&session->copy_fd), strerror(errno), errno);
         session->state = STREAM_SESSION_CLEANUP;
       }
       else {
@@ -883,17 +874,18 @@ _cb_parse_connection(int fd, void *data, bool event_read, bool event_write) {
  * @return -1 if an error happened, 0 otherwise
  */
 static int
-_cb_interface_listener(struct oonf_interface_listener *l) {
+_cb_interface_listener(struct os_interface_listener *interf) {
   struct oonf_stream_managed *managed;
   int result;
 
   /* calculate managed socket for this event */
-  managed = container_of(l, struct oonf_stream_managed, _if_listener);
+  managed = container_of(interf, struct oonf_stream_managed, _if_listener);
 
   result = _apply_managed(managed);
 
   OONF_DEBUG(LOG_STREAM,
-      "Result from interface triggered socket reconfiguration: %d", result);
+      "Result from interface %s triggered socket reconfiguration: %d",
+      interf->name, result);
 
   return result;
 }
